package main

import (
	"fmt"
	"io/ioutil"
	"log"
	"os"
	"r2mod-go/api"
	"r2mod-go/tools"
	"r2mod-go/utils"

	"github.com/fatih/color"
)

func main() {

	logFile, err := os.OpenFile(utils.SystemInfo.TmpDir+"/r2mod-go.log", os.O_RDWR|os.O_CREATE|os.O_APPEND, 0644)
	utils.CheckErr(err)

	defer logFile.Close()
	log.SetOutput(logFile)

	// Setup CLI switch

	var operation string
	if len(os.Args) >= 2 {
		operation = os.Args[1]
	} else {
		operation = "help"
	}

	// Color
	white := color.New(color.FgWhite).SprintFunc()

	// CLI Operations

	switch operation {
	case "help":
		fmt.Println(`Commands:
 > info -- get info about a mod
 > install, ins, i -- install mod(s)
 > remove, rem, r -- remove mod(s)
 > list, ls, li -- list mods
 > update, upgrade, up -- update mods and API cache
 > toggle, tm, togglemods -- toggle mods
 > pull -- forcefully update API cache, not mods
 > filter -- remove version numbers using regex from stdin
 `)
	case "info":
		if len(os.Args) <= 2 {
			fmt.Println("Usage: r2mod-go info <dependency string>")
		} else {
			var selectedmod api.Mod = api.GetModData(os.Args[2])
			color.Cyan("Mod Info: %s", os.Args[2])
			fmt.Println("  Name: " + selectedmod.Versions[0].FullName)
			fmt.Println("  Desc: " + selectedmod.Versions[0].Description)
			fmt.Println("  Version: " + selectedmod.Versions[0].VersionNumber)
			fmt.Println("  Download URL: " + selectedmod.Versions[0].DownloadURL)
		}
	case "install", "ins", "i":
		api.CheckAPICache()
		for _, m := range os.Args[2:] {

			fmt.Println(" >", tools.ExposeModString(m))
			stat, ov := tools.DownloadMod(tools.ExposeModString(m))
			if stat == 0 {
				color.Green("  > Already installed and up to date")
			} else if stat == 1 {
				color.Magenta("  > Updating to version %s", ov)
			} else {
				color.Blue("  > Downloaded")
			}
		}
	case "remove", "rem", "r":
		for _, m := range os.Args[2:] {
			fmt.Println(" >", tools.ExposeModString(m))

			if tools.RemoveMod(tools.ExposeModString(m)) == 1 {
				color.Red("  > Not installed")
			} else {
				color.Blue("  > Uninstalled")
			}
		}
	case "list", "ls", "li":
		files, err := ioutil.ReadDir(utils.SystemInfo.PluginDir)
		if err != nil {
			log.Fatal(err)
		}

		for i, f := range files {
			color.Blue("%3v %s", i, white(f.Name()))
		}
	case "update", "upgrade", "up":
		color.Cyan("> Checking API Cache")
		if api.CheckAPICache() == 0 {
			color.Green("> API Cache is up to date!")
		} else {
			color.Red("> API Cache out of date!")
			api.UpdateAPICache()
		}

		files, err := ioutil.ReadDir(utils.SystemInfo.PluginDir)
		if err != nil {
			log.Fatal(err)
		}

		for _, f := range files {
			fmt.Println(" >", tools.ExposeModString(f.Name()))
			stat, ov := tools.DownloadMod(tools.ExposeModString(f.Name()))
			if stat == 0 {
				color.Green("  > Already installed and up to date")
			} else if stat == 1 {
				color.Magenta("  > Updating to version %s", ov)
			} else {
				color.Blue("  > Downloaded")
			}

		}

		// color.Green("> Complete!")
	case "pull":
		color.Cyan("> Checking API Cache")
		if api.CheckAPICache() == 0 {
			color.Green("> API Cache is up to date!")
		} else {
			color.Red("> API Cache out of date!")
			api.UpdateAPICache()
		}
	case "toggle", "tm", "togglemods":
		if tools.ToggleMods() {
			color.Cyan("> Mods Enabled")
		} else {
			color.Cyan("> Mods Enabled")
		}
	case "filter":
		fmt.Println(tools.ExposeModString(os.Args[2]))
	default:
		fmt.Println("Unknown command, use 'r2go help' for a list of commands.")
	}
}
